package asm

import (
	"git.akyoto.dev/cli/q/src/register"
)

// Assembler contains a list of instructions.
type Assembler struct {
	instructions []Instruction
	labels       map[string]int
}

// New creates a new assembler.
func New() *Assembler {
	return &Assembler{
		instructions: make([]Instruction, 0, 8),
		labels:       map[string]int{},
	}
}

// Finalize generates the final machine code.
func (a *Assembler) Finalize() *Result {
	final := Result{}

	for _, instr := range a.instructions {
		instr.Write(&final.Code)
	}

	return &final
}

// AddLabel creates a new label at the current position.
func (a *Assembler) AddLabel(name string) {
	a.labels[name] = len(a.instructions)
}

// MoveRegisterNumber moves a number into the given register.
func (a *Assembler) MoveRegisterNumber(reg register.ID, number uint64) {
	a.instructions = append(a.instructions, Instruction{
		Mnemonic:    MOV,
		Destination: reg,
		Number:      number,
	})
}

// Syscall executes a kernel function.
func (a *Assembler) Syscall() {
	a.instructions = append(a.instructions, Instruction{
		Mnemonic: SYSCALL,
	})
}
